#! /bin/sh
#
# MaMa, the Make Master -- a Tcl/Tk script for handling Makefile operations.
#
# Copyright (C) 1998 Asko Seeba.
#
# This file is considered as a part of MaMa package.
#
# The copyright holder disclaims all responsibility or liability with
# respect to its usage or its effect upon hardware or computer
# systems, and maintains copyright as set out in the "LICENCE"
# document which accompanies distributions of MaMa v1.0a0 and upwards.

SRCDIR=src

SYSTEM=`uname -s`
# Solaris has different default 'id' commandline.
if [ "$SYSTEM" = SunOS ]
then
    ID=/usr/xpg4/bin/id
else
    ID=`which id`
fi

MAMAVER=1.0b0

# Variable FILES is used for checking if all needed files are on their right
# place in package. It is just a list of these filenames.
FILES="help.tcl mama.tcl title.tcl"

PLATFORMS="Linux SunOS IRIX"

echo_no_nl () {
    if [ "$SYSTEM" = SunOS -o "$SYSTEM" = IRIX ]
    then
	echo "$1\c"
    else
	echo -n $1
    fi
}

tested=n
for i in $PLATFORMS
do
    if [ "$SYSTEM" = $i ]
    then
	tested=y
	break
    fi
done

if [ $tested = n ]
then
    echo
    echo "NOTE! You are using platform $SYSTEM. Setup of current MaMa release"
    echo "isn't tested on it. I try to act like on RedHat Linux. So, You'll"
    echo "have quite a good possibility to see strange behavior or even fatal"
    echo "errors somewhere now."
fi

echo
echo_no_nl "Searching for 'wishx' in Your PATH... "
WISH=`which wishx`
# I keep this line for debuging purposes only.
#WISH=

if [ -z "$WISH" ]
then
    echo "Not found!"
    echo
    echo "Can't find a Tcl/Tk interpretator called 'wishx'. If You don't"
    echo "understand what I'm talking about, press Ctrl+c or type 'q' and ask"
    echo "from Your system administrator about it. If You are the system"
    echo "administrator and You know that You don't have 'wishx', point Your"
    echo "web browser to http://www.tcltk.com/. 'wishx' stands for 'extended"
    echo "wish'. If You know that You have wishx, then type here the name of"
    echo "it with path to it in form /foo/bar/wishx where '/foo/bar/' is the"
    echo "location of 'wishx' and 'wishx' is the name of 'wishx' :). NOTE!"
    echo "There are two different programs: 'wish' and 'wishx'. 'wish' isn't"
    echo "the proper one. 'wishx' is needed. So,"
    while [ -z "$WISH" ]
    do
	echo_no_nl "Your 'wish' ('q' for quit): "
	read WISH
	if [ -n "$WISH" ]
	then
	    if [ "$WISH" = q ]
	    then
		exit 0
	    fi
	    if [ -x "$WISH" ]
	    then
		break
	    fi
	    if [ -d "$WISH" ]
	    then
		echo "$WISH is a directory"
		WISH=
		continue
	    fi
	    if [ ! -f "$WISH" ]
	    then
		echo "File named $WISH doesn't exist"
		WISH=
		continue
	    fi
	    echo "$WISH is not an executable file"
	    WISH=
	fi
    done
else
    echo "Found: $WISH!"
fi

cd $SRCDIR
OK=1
for i in $FILES
do
    if [ ! -f "$i" ]
    then
	echo
	echo "didn't find a file '$i'".
	echo "If You are not in the root directory of the 'MaMa' package i.e."
	echo "the directory where the 'setup.sh' should be located after"
	echo "uncompressing and untaring the package, then 'cd' to this"
	echo "directory, run setup again and You can stop reading HERE."
	echo
	echo "If You already are in this directory then it's possible that"
	echo "something is messed up in the package and if it is messed up"
	echo "after unpacking it. You have to get the package from Your"
	echo "nearest possible site once more. If it is a fault of package"
	echo "then it is quite likely my fault. If it is the latest version of"
	echo "the package, let me know about it, if not, get the latest."
	echo

	exit 1
    fi
done

echo_no_nl "Checking Your identity... "
IDENTITY=`$ID -un`
echo "$IDENTITY!"

if [ "$IDENTITY" = root ]
then
    echo
    echo "Looks like You are system administrator. Thus, I guess You want to"
    echo "install the 'Make Master' as a system-wide software. I set the"
    echo "default values according to this assumption. Don't worry! I'll ask"
    echo "You if You want to change these settings."

    BINDEST=/usr/local/bin
    # Default permission for accessing executable and it's directory (if
    # directory is not created yet).
    BINPERM=755
    BINDESTPERM=755

    LIBDIR=/usr/local/lib
    LIBDEST=$LIBDIR/mama$MAMAVER
    # Default permission for accessing libraries and their directory (if
    # directory is not created yet).
    LIBPERM=644
    LIBDESTPERM=755

    RCDEST=/usr/local/etc
    RCFILE=mama.rc
    # Default permission for accessing configuration file and it's directory
    # (if directory is not created yet).
    RCPERM=644
    RCDESTPERM=755
else
    echo
    echo "Looks like You don't have system administrator's priviledges just"
    echo "now. Thus, I guess You want to install the 'Make Master' to Your"
    echo "home directory. I set the default values according to this"
    echo "assumption. Don't worry! I'll ask You if You want to change these"
    echo "settings, but You still have to take into account that You cant"
    echo "install the 'Make Master' to the directories You don't have a"
    echo "permission to fo it."

    BINDEST=$HOME/bin
    # The line here is for testing purposes...
    #BINDEST=askdj/adkh/wekfj/wekfjh
    # Default permission for accessing executable and it's directory (if
    # directory is not created yet).
    BINPERM=700
    BINDESTPERM=700

    LIBDIR=$HOME/lib
    LIBDEST=$LIBDIR/mama$MAMAVER
    # Default permission for accessing libraries and their directory (if
    # directory is not created yet).
    LIBPERM=600
    LIBDESTPERM=700

    RCDEST=$HOME
    RCFILE=.mamarc
    # Default permission for accessing configuration file and it's directory
    # (if directory is not created yet).
    RCPERM=600
    RCDESTPERM=700
fi

# Function that asks for directory to install something.
# Syntax: ask_dir <default dir> <progname> <explain> <permission>
# Result: sets the variable DEST to be the directory to where the program
#        named <progname> with explanation <explain> will be installed. If
#        directory doesn't exist, it will be installed with permission
#        <permission>
ask_dir () {
    ask_ok=0

    while [ "$ask_ok" -eq 0 ]
    do
	echo
	echo "Where do You want to install Your '$2' -- the $3?"
	echo_no_nl "[Default is $1]>"
	read ask_answer
	if [ -z "$ask_answer" ]
	then
	    ask_answer=$1
	fi
	if [ ! -d "$ask_answer" ]
	then
	    ask_yes_or_no=
	    while [ -z "$ask_yes_or_no" ]
	    do
		echo
		echo "Directory $ask_answer doesn't exist."
		echo_no_nl "Create it (y/n)? "
		read ask_yes_or_no
		if [ -z "$ask_yes_or_no" ]
		then
		    continue
		fi
		if [ "$ask_yes_or_no" = "y" ]
		then
		    if [ "$SYSTEM" = IRIX ]
		    then
			install -m $4 -dir $ask_answer
		    else
		    	install -m $4 -d $ask_answer
		    fi
		    if [ "$?" -ne 0 ]
		    then
			echo "Creation of directory $ask_answer failed"
		    else
			echo "Directory $ask_answer created with access"
			echo "permissions $4. You can change these permissions"
			echo "if You don't like these ones and You know how to"
			echo "do it."
			DEST=$ask_answer
			ask_ok=1
		    fi
		    break
		fi
		if [ "$ask_yes_or_no" = "n" ]
		then
		    break
		fi
		echo
		echo "Beg Your bard'n?"
		# We set it to empty string because when ask_yes_or_no is empty
		# string the yes or no will be asked again.
		ask_yes_or_no=
	    done
	elif [ ! -d "$ask_answer" ]
	then
	    echo "$ask_answer is not a directory"
	elif [ ! -w "$ask_answer" ]
	then
	    echo "You don't have a permission to install things into directory"
	    echo "$ask_answer."
	else
	    ask_ok=1
	fi
	if [ "$ask_ok" -eq 1 -a -n "$ask_answer" ]
	then
	    DEST=$ask_answer
	fi
	if [ -z "$ask_answer" ]
	then
	    ask_ok=1
	fi
    done
} 

ok=0
while [ "$ok" -eq 0 ]
do
    ask_dir $BINDEST mama "Make Master executable" $BINDESTPERM
    BINDEST=$DEST

    ask_dir $LIBDEST libraries "MaMa's library files" $LIBDESTPERM
    LIBDEST=$DEST

    echo "Now I'll ask You for location of MaMa's rc (initianlization) file."
    echo "Don't worry about an old rc file if You have it configured according"
    echo "to Your own needs. If I'll find an old rc file at the location You"
    echo "want to install new one, I'll give You an option to include it's"
    echo "contents to new one. You can edit the settings in new rc file later"
    echo "to verify thing are ok."
    ask_dir $RCDEST $RCFILE "MaMa configuration file" $RCDESTPERM
    RCDEST=$DEST

    if [ -f "$RCDEST/$RCFILE" ]
    then
	echo
	echo "I found an old configuration file: $RCDEST/$RCFILE."
	add_old_rc=
	while [ -z "$add_old_rc" ]
	do
	    echo_no_nl "Add an old $RCFILE contents to new one (y/n)? "
	    read add_old_rc
	    if [ -z "$add_old_rc" ]
	    then
		continue
	    fi
	    if [ "$add_old_rc" = y ]
	    then
		break
	    fi
	    if [ "$add_old_rc" = n ]
	    then
		break
	    fi
	    echo
	    echo "Beg Your bard'n?"
	    # We set it to empty string because when add_old_rc is empty
	    # string the yes or no will be asked again.
	    add_old_rc=
	done
    fi

    yes_or_no=
    while [ -z "$yes_or_no" ]
    do
	echo
	echo "Make Master executable goes to $BINDEST."
	echo "Make Master libraries go to $LIBDEST."
	echo "Make Master configuration file goes to $RCDEST."
	if [ "$add_old_rc" = y ]
	then
	    echo "Old configuration file will be added to new one."
	fi
	if [ "$add_old_rc" = n ]
	then
	    echo "Old configuration file will not be added to new one."
	fi
	echo_no_nl "Is it ok (y/n)? "
	read yes_or_no
	if [ -z "$yes_or_no" ]
	then
	    continue
	fi
	if [ "$yes_or_no" = y ]
	then
	    ok=1
	    break
	fi
	if [ "$yes_or_no" = n ]
	then
	    break
	fi
	echo
	echo "Beg Your bard'n?"
	# We set it to empty string because when yes_or_no is empty
	# string the yes or no will be asked again.
	yes_or_no=
    done
done

#
# Now the annoying operations are over. Following is more interesting.
#

echo
echo "Going to build and install MaMa version $MAMAVER."
MAMA=mama.$MAMAVER

echo
echo_no_nl "Creating Makefile... "

if [ -f Makefile ]
then
    rm -f Makefile
fi
echo "SHELL=/bin/sh" > Makefile
echo "WISH=$WISH" >> Makefile
echo "MAMAVER=$MAMAVER" >> Makefile
echo 'MAMA=mama.$(MAMAVER)' >> Makefile
echo "BINDEST=$BINDEST" >> Makefile
echo "BINPERM=$BINPERM" >> Makefile
echo "RCDEST=$RCDEST" >> Makefile
echo "RCPERM=$RCPERM" >> Makefile
echo "LIBDIR=$LIBDIR" >> Makefile
echo "LIBDEST=$LIBDEST" >> Makefile
echo "LIBPERM=$LIBPERM" >> Makefile
echo "RCFILE=$RCFILE" >> Makefile
echo "FILES=$FILES" >> Makefile
if [ "$SYSTEM" = SunOS ]
then
    echo 'INSTALL_BIN=install -c $(BINDEST) -m $(BINPERM) $(MAMA)' >> Makefile
    echo 'INSTALL_HELP=install -c $(LIBDEST) -m $(LIBPERM) help.tcl' >> Makefile
    echo 'INSTALL_RC=install -c $(RCDEST) -m $(RCPERM) $(RCFILE)' >> Makefile
elif [ "$SYSTEM" = IRIX ]
then
    echo 'INSTALL_BIN=install -m $(BINPERM) -f $(BINDEST) $(MAMA)' >> Makefile
    echo 'INSTALL_HELP=install -m $(LIBPERM) -f $(LIBDEST) help.tcl' >> Makefile
    echo 'INSTALL_RC=install -m $(RCPERM) -f $(RCDEST) $(RCFILE)' >> Makefile
else
    echo 'INSTALL_BIN=install -m $(BINPERM) $(MAMA) $(BINDEST)' >> Makefile
    echo 'INSTALL_HELP=install -m $(LIBPERM) help.tcl $(LIBDEST)' >> Makefile
    echo 'INSTALL_RC=install -m $(RCPERM) $(RCFILE) $(RCDEST)' >> Makefile
fi
echo >> Makefile
tail +12 Makefile.src >> Makefile
echo "Done!"

echo "Cleaning up the directory..."
echo
make clean
echo
echo "Cleaned!"

echo
echo_no_nl "Creating configuration file... "
if [ -f "$RCFILE" ]
then
    rm -f $RCFILE
fi
echo "#" >> $RCFILE
echo "# Whatever here is." > $RCFILE
echo "#" >> $RCFILE

echo >> $RCFILE
echo "#" >> $RCFILE
echo "# MAMA_LIB_DIR specifies where all files needed by MaMa are" >> $RCFILE
echo "# located except rc-file." >> $RCFILE
echo "#" >> $RCFILE
echo "set MAMA_LIB_DIR $LIBDEST" >> $RCFILE
echo >> $RCFILE
echo >> $RCFILE
if [ "$IDENTITY" = 'root' ]
then
    echo '#' >> $RCFILE
    echo '# Everything defined in this rc-file should go before' >> $RCFILE
    echo '# next three lines because then user has possibility to' >> $RCFILE
    echo '# override these settings with his/her own settings.' >> $RCFILE
    echo '#' >> $RCFILE
    echo 'set USER_RC $env(HOME)/.mamarc' >> $RCFILE
    echo 'if [file exists $USER_RC] {' >> $RCFILE
    echo '    source $USER_RC' >> $RCFILE
    echo '}' >> $RCFILE
fi

if [ "$add_old_rc" = y ]
then
    echo >> $RCFILE
    echo >> $RCFILE
    echo "#" >> $RCFILE
    echo "# Here comes an old rc file contents found during" >> $RCFILE
    echo "# installation of MaMa." >> $RCFILE
    echo "#" >> $RCFILE
    echo >> $RCFILE
    cat $RCDEST/$RCFILE >> $RCFILE
fi

echo "Done!"
echo
echo "Now making Your MaMa..."
echo
make $MAMA
if [ "$?" -ne 0 ]
then
    echo
    echo "Making MaMa generated an error. Giving up..."
    echo
    exit 1
fi
echo
echo "MaMa is completed."
echo
echo "Now installing Your MaMa..."
echo
make install
if [ "$?" -ne 0 ]
then
    echo
    echo "Installation of MaMa generated an error. Giving up..."
    echo
    exit 1
fi
echo
echo "MaMa version $MAMAVER is ready to rock!"
echo "++++++++++++++++++++++++++++++++++++"
echo
echo "To run MaMa, type"
echo
echo "	mama"
echo
echo "at Your command line prompt."
echo
echo "NOTE: I didn't removed an old binary(s) from"
echo "      $BINDEST and libary(s) from $LIBDIR."
echo "      You can delete them safely, if You don't need"
echo "      them any more."
echo

exit 0
